/*
 * Copyright 2005-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package org.wamblee.system.adapters;

import org.wamblee.system.core.RequiredInterface;
import org.wamblee.system.core.Scope;

import java.util.ArrayList;
import java.util.List;

/**
 * General configuration for an instantiated object.
 * 
 * @author Erik Brakkee
 */
public class ObjectConfiguration {
    private Class clazz;

    private SetterConfiguration setterConfig;

    /**
     * Creates a new ObjectConfiguration object.
     * 
     */
    public ObjectConfiguration(Class aClass) {
        clazz = aClass;
        setterConfig = new SetterConfiguration(aClass);
    }

    /**
     * Performs injection into an object of the configured class using
     * information from the given scope.
     * 
     * @param aScope
     *            Scope.
     * @param aObject
     *            Object.
     */
    public void inject(Scope aScope, Object aObject) {
        setterConfig.inject(aScope, aObject);
    }

    public SetterConfiguration getSetterConfig() {
        return setterConfig;
    }

    public List<RequiredInterface> getRequiredInterfaces() {
        List<RequiredInterface> result = new ArrayList<RequiredInterface>();
        result.addAll(setterConfig.getRequiredInterfaces());

        return result;
    }

    public boolean appliesTo(Object aObject) {
        return clazz.isInstance(aObject);
    }
}
