/*
 * Copyright 2005-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package org.wamblee.system.adapters;

import org.wamblee.system.core.RequiredInterface;
import org.wamblee.system.core.Scope;

/**
 * Value provider that provides a value through a required interface.
 * 
 * @author Erik Brakkee
 */
class RequiredInterfaceProvider implements ValueProvider {
    private RequiredInterface required;

    /**
     * Constructs the provider
     * 
     * @param aRequired
     *            Required interface.
     */
    public RequiredInterfaceProvider(RequiredInterface aRequired) {
        required = aRequired;
    }

    @Override
    public Object getValue(Scope aScope) {
        return aScope.getInterfaceImplementation(required.getProvider(),
            Object.class);
    }

    public RequiredInterface getRequiredInterface() {
        return required;
    }
}
