/*
 * Copyright 2005-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package org.wamblee.system.core;

import java.util.Collection;

/**
 * Represents an interface provided by a component. Different component objects
 * should never share ProvidedInterface instances!
 * 
 * @author Erik Brakkee
 */
public interface ProvidedInterface extends NamedInterface {
    /**
     * Returns the service type.
     * 
     * @return Service type.
     */
    Class[] getInterfaceTypes();

    /**
     * Determines whether the current provided interface exceeds the given
     * provided interface. In other words if it can provide at least what the
     * given provided interface can provide.
     * 
     * @param aInterface
     *            Interface to compare to.
     * 
     * @return True if the current interface exceeds the given provided
     *         interface.
     */
    boolean covers(ProvidedInterface aInterface);
}
