/*
 * Copyright 2005-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package org.wamblee.system.graph.component;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.wamblee.system.core.SystemAssemblyException;
import org.wamblee.system.graph.Edge;
import org.wamblee.system.graph.Graph;
import org.wamblee.system.graph.Node;
import org.wamblee.system.graph.Visitor;

import java.util.List;

/**
 * Visitor that checks whether all required external interfaces of the container
 * are provided.
 * 
 * @author Erik Brakkee
 */
public class CheckExternallyRequiredVisitor implements Visitor {
    private static final Log LOG = LogFactory.getLog(CheckExternallyRequiredVisitor.class);

    private Graph graph;

    /**
     * Creates a new CheckExternallyRequiredVisitor object.
     * 
     */
    public CheckExternallyRequiredVisitor(Graph aGraph) {
        graph = aGraph;
    }

    @Override
    public void visitEdge(Edge aEdge) {
        // Empty.
    }

    @Override
    public void visitNode(Node aNode) {
        if (aNode instanceof ExternalRequiredInterfaceNode) {
            ExternalRequiredInterfaceNode required = (ExternalRequiredInterfaceNode) aNode;

            if (!required.getRequired().isOptional() &&
                (required.getRequired().getProvider() == null)) {
                throw new SystemAssemblyException(aNode +
                    ": External required interface is not provided");
            }

            List<Edge> edges = graph.findIncoming(aNode);

            if (edges.isEmpty()) {
                LOG.warn(aNode + ": Superfluous required interface");
            }

            for (Edge edge : edges) {
                Node from = edge.getFrom();
                assert from instanceof RequiredInterfaceNode;

                RequiredInterfaceNode reqNode = (RequiredInterfaceNode) from;

                if (!reqNode.getRequired().isOptional() &&
                    required.getRequired().isOptional()) {
                    throw new SystemAssemblyException(
                        aNode +
                            ": externally required interface is optional but a corresponding internal required interface is mandatory: " +
                            reqNode);
                }
            }
        }
    }
}
